<?php

namespace App\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Security\Http\Authentication\AuthenticationUtils;
use Symfony\Component\HttpFoundation\Request;
use App\Service\CguUserService;

class SecurityController extends AbstractController
{
    private CguUserService $cguUserService;

    public function __construct(CguUserService $cguUserService)
    {
        $this->cguUserService = $cguUserService;
    }

    #[Route(path: '/login', name: 'app_login')]
    public function login(AuthenticationUtils $authenticationUtils): Response
    {
        // get the login error if there is one
        $error = $authenticationUtils->getLastAuthenticationError();

        // last username entered by the user
        $lastUsername = $authenticationUtils->getLastUsername();

        

        return $this->render('security/login.html.twig', [
            'last_username' => $lastUsername,
            'error' => $error,
        ]);
    }

    #[Route(path: '/logout', name: 'app_logout')]
    public function logout(): void
    {
        throw new \LogicException('This method can be blank - it will be intercepted by the logout key on your firewall.');
    }

    #[Route(path: '/consent', name: 'app_consent')]
    public function consent(Request $request): Response
    {
        // Handle form submission
        if ($request->isMethod('POST')) {
            // Check if user declined consent
            if (!$request->request->has('decline')) {
                // User accepted the CGU, save this in the database
                $this->cguUserService->acceptLatestCgu($this->getUser());
            }
            
            // Redirect back to the OAuth authorization endpoint with all the query parameters
            return $this->redirectToRoute('oauth2_authorize', $request->query->all());
        }
        
        // For GET requests, just show the consent form
        return $this->render('security/consent.html.twig');
    }
}
