<?php

namespace App\EventSubscriber;

use League\Bundle\OAuth2ServerBundle\Event\AuthorizationRequestResolveEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use Symfony\Bundle\SecurityBundle\Security;
use Symfony\Component\Security\Core\User\UserInterface;
use Symfony\Component\Security\Http\FirewallMapInterface;
use Symfony\Component\Security\Http\Util\TargetPathTrait;
use App\Service\CguUserService;

class AuthorizationCodeSubscriber implements EventSubscriberInterface
{
    use TargetPathTrait;

    private Security $security;
    private UrlGeneratorInterface $urlGenerator;
    private RequestStack $requestStack;
    private $firewallName;
    private $cguUserService;

    public function __construct(Security $security, UrlGeneratorInterface $urlGenerator, RequestStack $requestStack, FirewallMapInterface $firewallMap, CguUserService $cguUserService)
    {
        $this->security = $security;
        $this->urlGenerator = $urlGenerator;
        $this->requestStack = $requestStack;
        $this->firewallName = $firewallMap->getFirewallConfig($requestStack->getCurrentRequest())->getName();
        $this->cguUserService = $cguUserService;
    }

    public function onLeagueOauth2ServerEventAuthorizationRequestResolve(AuthorizationRequestResolveEvent $event): void
    {
        $request = $this->requestStack->getCurrentRequest();
        $user = $this->security->getUser();
        
        // Default response is to redirect to login if not logged in
        $response = new RedirectResponse($this->urlGenerator->generate('app_login'), 307);
        
        if (!$user instanceof UserInterface) {
            // Save the target path and redirect to login
            $this->saveTargetPath($request->getSession(), $this->firewallName, $request->getUri());
            $event->setResponse($response);
            return;
        }
        
        // User is logged in, check if they've accepted the latest CGU
        if (!$this->cguUserService->isLatestCguAccepted($user)) {
            // Redirect to consent page with all query parameters
            $response = new RedirectResponse($this->urlGenerator->generate('app_consent', $request->query->all()), 307);
            $event->setResponse($response);
            return;
        }
        
        // User has accepted CGU, authorize the request
        $event->resolveAuthorization(true);
    }

    public static function getSubscribedEvents(): array
    {
        return [
            'league.oauth2_server.event.authorization_request_resolve' => 'onLeagueOauth2ServerEventAuthorizationRequestResolve',
        ];
    }
}